"""
The MIT License (MIT)

Copyright (c) 2020-Present AbstractUmbra

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""
from __future__ import annotations

import json
from typing import Optional

import aiohttp

from .constants import API_BASE_URL, CLIENT_TIMEOUT, MB_URL_RE
from .errors import APIError, BadPasteID
from .objects import Paste, PasteData


try:
    import requests
except ImportError:
    HAS_REQUESTS = False
else:
    HAS_REQUESTS = True

__all__ = (
    "Client",
    "SyncClient",
)


class Client:
    """
    Client for interacting with the Mystb.in API.

    Attributes
    ----------
    session: Optional[Union[:class:`aiohttp.ClientSession`, :class:`requests.Session`]]
        Optional session to be passed to the creation of the client.
    """

    __slots__ = ("session",)

    def __init__(
        self,
        *,
        session: Optional[aiohttp.ClientSession] = None,
    ) -> None:
        self.session: Optional[aiohttp.ClientSession] = session

    async def _generate_async_session(self) -> None:
        """
        This method will create a new and blank `aiohttp.ClientSession` instance for use.
        This method should not be called if a session is passed to the constructor.
        """
        self.session = aiohttp.ClientSession()

    async def post(self, content: str, syntax: Optional[str] = None) -> Paste:
        """
        This will post to the Mystb.in API and return the url.
        Can pass an optional suffix for the syntax highlighting.

        Parameters
        -----------
        content: :class:`str`
            The content you are posting to the Mystb.in API.
        syntax: Optional[:class:`str`]
            The optional suffix to append the returned URL which is used for syntax highlighting on the paste.
        """
        if not self.session:
            await self._generate_async_session()

        assert self.session is not None

        multi_part_write = aiohttp.MultipartWriter()
        paste_content = multi_part_write.append(content)
        paste_content.set_content_disposition("form-data", name="data")
        paste_content = multi_part_write.append_json({"meta": [{"index": 0, "syntax": syntax}]})
        paste_content.set_content_disposition("form-data", name="meta")

        async with self.session.post(
            API_BASE_URL,
            data=multi_part_write,
            timeout=aiohttp.ClientTimeout(CLIENT_TIMEOUT),
        ) as response:
            status_code = response.status
            response_text = await response.text()

            if not 200 <= status_code < 300:
                raise APIError(status_code, response_text)

            response_data = await response.json()

        return Paste(response_data, syntax)

    async def get(self, paste_id: str) -> PasteData:
        """
        This will perform a GET request against the Mystb.in API and return the url.
        Must be passed a valid paste ID or URL.

        Parameters
        ----------
        paste_id: :class:`str`
            The ID of the paste you are going to retrieve.
        """
        paste_id_match = MB_URL_RE.match(paste_id)

        if not paste_id_match:
            raise BadPasteID("This is an invalid Mystb.in paste ID.", None)

        paste_id = paste_id_match.group("ID")

        if not self.session:
            await self._generate_async_session()

        assert self.session is not None

        async with self.session.get(f"{API_BASE_URL}/{paste_id}", timeout=aiohttp.ClientTimeout(CLIENT_TIMEOUT)) as response:
            if not 200 <= response.status < 300:
                raise BadPasteID("This is an invalid Mystb.in paste ID.", response)
            paste_data = await response.json()

        return PasteData(paste_id, paste_data)

    async def close(self) -> None:
        """Async only - close the session."""
        if self.session and isinstance(self.session, aiohttp.ClientSession):
            await self.session.close()


class SyncClient:
    def __init__(self, *, session: Optional[requests.Session] = None) -> None:
        if HAS_REQUESTS is not True:
            raise RuntimeError("The `requests` addon is required to use the Sync client.")

        self.session: requests.Session = session or requests.Session()

    def post(self, content: str, syntax: Optional[str] = None) -> Paste:
        payload = {"meta": [{"index": 0, "syntax": syntax}]}

        response: requests.Response = self.session.post(
            API_BASE_URL,
            files={
                "data": content,
                "meta": (None, json.dumps(payload), "application/json"),
            },
            timeout=CLIENT_TIMEOUT,
        )

        if 200 <= response.status_code < 300:
            raise APIError(response.status_code, response.text)

        return Paste(response.json(), syntax)

    def get(self, paste_id: str) -> PasteData:
        paste_id_match = MB_URL_RE.match(paste_id)

        if not paste_id_match:
            raise BadPasteID("This is an invalid Mystb.in paste ID.", None)

        paste_id = paste_id_match.group("ID")

        with self.session.get(f"{API_BASE_URL}/{paste_id}", timeout=CLIENT_TIMEOUT) as response:
            if not 200 <= response.status_code < 300:
                raise BadPasteID("This is an invalid Mystb.in paste ID.", response)

        paste_data = response.json()
        return PasteData(paste_id, paste_data)



"""
The MIT License (MIT)

Copyright (c) 2020-Present AbstractUmbra

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""

import datetime
from textwrap import dedent
from typing import Any, Optional

from .constants import PASTE_BASE


class Paste:
    """
    A class representing the return data from the API after performing a POST request.

    Attributes
    ----------
    paste_id: :class:`str`
        The ID returned from the API. Genertally it is 3 random choice English words.
    nick: :class:`str`
        The nickname you requested the paste be named.
    syntax: :class:`str`
        The syntax (or syntax highlighting) you requested when creating the Paste. Returns as a suffix on the URL.
    """

    __slots__ = ("paste_id", "nick", "syntax")

    def __init__(self, json_data: dict[str, Any], syntax: Optional[str] = None):
        self.paste_id: str = json_data["pastes"][0]["id"]
        self.nick: Optional[str] = json_data["pastes"][0]["nick"]
        self.syntax: Optional[str] = syntax

    def __str__(self) -> str:
        return self.url

    def __repr__(self) -> str:
        return f"<Paste id={self.paste_id} nick={self.nick} syntax={self.syntax}>"

    @property
    def url(self) -> str:
        """:class:`str`: Returns the formatted url of ID and syntax."""
        syntax = f".{self.syntax}" if self.syntax else ""
        return PASTE_BASE.format(self.paste_id, syntax)

    def with_syntax(self, new_syntax: Optional[str]) -> str:
        """
        Changes the syntax of the current Paste to `new_syntax`

        Parameters
        ----------
        new_syntax: :class:`str`
            The new suffix to append to the Paste.
        """
        new_syntax = f".{new_syntax}" if new_syntax else None
        return PASTE_BASE.format(self.paste_id, new_syntax)


class PasteData:
    """
    A class representing the return data from the API after performing a GET request.

    Attributes
    ----------
    paste_id: :class:`str`
        The ID you wish to retrieve from the API.
    paste_content: :class:`str`
        The content returned from the paste.
    paste_syntax: :class:`str`
        The syntax you specified that this Paste is in.
    paste_nick: Optional[:class:`str`]
        The nick set for this paste on the API.
    """

    __slots__ = (
        "paste_id",
        "_paste_data",
        "paste_content",
        "paste_syntax",
        "paste_nick",
        "_created_at",
    )

    def __init__(self, paste_id: str, paste_data: dict[str, Any]):
        self.paste_id: str = paste_id
        self._paste_data: dict[str, Any] = paste_data
        self.paste_content: str = paste_data["data"]
        self.paste_syntax: str = paste_data["syntax"]
        self.paste_nick: Optional[str] = paste_data["nick"]
        self._created_at: str = paste_data["created_at"]

    def __str__(self) -> str:
        return self.content

    def __repr__(self) -> str:
        return f"<PasteData id={self.paste_id!r} nick={self.paste_nick!r} syntax={self.paste_syntax!r}>"

    @property
    def url(self) -> str:
        """:class:`str`: The Paste ID's URL."""
        syntax = f".{self.paste_syntax}" if self.paste_syntax else ""
        return PASTE_BASE.format(self.paste_id, syntax)

    @property
    def created_at(self) -> datetime.datetime:
        """:class:`datetime.datetime`: Returns a UTC datetime of when the paste was created."""
        return datetime.datetime.strptime(self._created_at, "%Y-%m-%dT%H:%M:%S.%f")

    @property
    def content(self) -> str:
        """:class:`str`: Return the paste content but dedented correctly."""
        return dedent(self.paste_content)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mystbin']

package_data = \
{'': ['*']}

modules = \
['py']
install_requires = \
['aiohttp>=3.7.4,<4.0.0']

extras_require = \
{'docs': ['sphinx>=4.0.0,<5.0.0', 'sphinxcontrib-trio', 'furo'],
 'requests': ['requests>=2.24.0,<3.0.0']}

entry_points = \
{'console_scripts': ['version = mystbin.__main__:show_version']}

setup_kwargs = {
    'name': 'mystbin.py',
    'version': '4.0.2',
    'description': 'A small simple wrapper around the mystb.in API.',
    'long_description': '<div align="center">\n    <h1>Mystbin.py!</h1>\n    <a href=\'https://mystbinpy.readthedocs.io/en/latest/?badge=latest\'>\n        <img src=\'https://readthedocs.org/projects/mystbinpy/badge/?version=latest\' alt=\'Documentation Status\' />\n    </a>\n    <a href=\'https://github.com/AbstractUmbra/mystbin.py/workflows/Code%20Linting\'>\n        <img src=\'https://github.com/AbstractUmbra/mystbin.py/workflows/Code%20Linting/badge.svg?branch=main\' alt=\'Linting status\' />\n    </a>\n    <a href=\'https://github.com/AbstractUmbra/mystbin.py/workflows/Build\'>\n        <img src=\'https://github.com/AbstractUmbra/mystbin.py/workflows/Build/badge.svg\' alt=\'Build status\' />\n    </a>\n</div>\n<br>\n\nA small simple wrapper around the [MystB.in](https://mystb.in/) API.\n----------\n### Features\n\n- [x] - `POST`ing to the API, which will return the provided url.\n- [x] - `GET`ting from the API, provided you know the URL or paste ID.\n- [ ] - `DELETE`ing from the API, provided the paste is attached to your account.\n- [ ] - `PATCH`ing to the API, provided the paste is attached to your account.\n- [x] - Ability to pass in a sync or async session / parameter so it is flexible.\n- [x] - Write a real underlying Client for this, it will be required for...\n- [ ] - ... Authorization. Awaiting the API making this public as it is still WIP.\n\n### Installation\nThis project will be on [PyPI](https://pypi.org/project/mystbin.py/) as a stable release, you can always find that there.\n\nInstalling via `pip`:\n```shell\npython -m pip install -U mystbin.py\n# or for optional sync addon...\npython -m pip install -U mystbin.py[requests]\n```\n\nInstalling from source:\n```shell\npython -m pip install git+https://github.com/AbstractUmbra/mystbin-py.git #[requests] for sync addon\n```\n\n### Usage examples\nSince the project is considered multi-sync, it will work in a sync/async environment, see the optional dependency of `requests` below.\n\n```py\n# async example - it will default to async\nimport mystbin\n\nmystbin_client = mystbin.Client()\n\npaste = await mystbin_client.post("Hello from MystBin!", syntax="python")\nstr(paste)\n>>> \'https://mystb.in/<your generated ID>.python\'\n\npaste.url\n>>> \'https://mystb.in/<your generated ID>.python\'\n\nget_paste = await mystbin_client.get("https://mystb.in/<your generated ID>")\nstr(get_paste)\n>>> "Hello from MystBin!"\n\npaste.created_at\n>>> datetime.datetime(2020, 10, 6, 10, 53, 57, 556741)\n```\n\n```py\nimport mystbin\n\nmystbin_client = mystbin.SyncClient()\n\npaste = mystbin_client.post("Hello from sync Mystb.in!", syntax="text")\nstr(paste)\n>>> \'https://mystb.in/<your generated ID>.text\'\n```\n\nNOTE: There is a timeout of 15s for each operation.\n',
    'author': 'AbstractUmbra',
    'author_email': 'Umbra@AbstractUmbra.xyz',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/AbstractUmbra/mystbin-py',
    'packages': packages,
    'package_data': package_data,
    'py_modules': modules,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



"""
The MIT License (MIT)

Copyright (c) 2020-Present AbstractUmbra

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""

from __future__ import annotations


__all__ = (
    "BadPasteID",
    "MystbinException",
    "APIError",
)

from typing import Optional, Union

import aiohttp


try:
    import requests
except ModuleNotFoundError:
    pass


class MystbinException(Exception):
    """Error when interacting with Mystbin."""

    def __init__(self, /, message: str, response: Optional[Union[aiohttp.ClientResponse, requests.Response]]) -> None:
        self.message: str = message
        self.response: Optional[Union[aiohttp.ClientResponse, requests.Response]] = response


class BadPasteID(MystbinException):
    """Bad Paste ID."""


class APIError(MystbinException):
    """
    Exception relationg to the API of Mystbin.

    Attributes
    ----------
    status_code: :class:`int`
        The HTTP Status code return from the API.
    message: :class:`str`
        The Message supplied with the HTTP status code from the API.
    """

    __slots__ = ("status_code", "message")

    def __init__(self, status_code: int, message: str):
        self.status_code: int = status_code
        self.message: str = message

    def __repr__(self) -> str:
        return f"<MystbinError status_code={self.status_code} message={self.message}>"

    def __str__(self) -> str:
        return self.message
